# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

#NOTE THIS LINE!!!
from query_term_feature_le import *
from Constants import *
import subprocess
import os
import glob
import sys
from time import strftime

def trecid2qids(trecid):
    if trecid>=13:
        trecid+=1
    
    return range(trecid*50+1,trecid*50+51)

def qid2trecid(qid):
    if qid>700:
        qid-=50

    return (qid-1)/50

# NOTE
# strid should only be "train" , "test", or "dev"
def prepare_svm_file(qids, fbdoc, svddim, run_tag, strid, optional_str = '', nsample = None):
    oddp_tag = '_oddp' if '.oddp' in run_tag else ''
    prob_tag = ' -b 1 ' if '.oddp' not in run_tag else ' '
    nsample_tag = '.nsample.%d' % nsample if not nsample == None else ''
    qids.sort()
    svmname = None
    if qids[-1]-qids[0]+1 != len(qids):
        for index in xrange(len(qids)):
            if qids[index]-qids[0]!=index:
                svmname = 'q%d-%d-%d.%d.desc.fbdoc%d.svddim%d%s%s'%(qids[0],qids[index],qids[-1], len(qids),fbdoc,svddim,run_tag,nsample_tag)
                break
    else:
        svmname = 'q%d-%d.%d.desc.fbdoc%d.svddim%d%s%s'%(qids[0],qids[-1], len(qids),fbdoc,svddim,run_tag, nsample_tag)
    
    # read examples
    ofilename = '../tmp/%s.%s%s.svm'%(svmname, strid, optional_str)
    ofile=open(ofilename,'w')
    for qid in qids:
        trecid = qid2trecid(qid)

        # NOTE THE MU HERE!
#        mu = mus[trecid]
        mu=900
        if trecid ==9 or trecid==10:
            mu=1500

        mu=2500 # FOR SIGIR
        ifilename = ''
        if run_tag == '' or run_tag == '.oddp':
            ifilename = '../qtfs%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.svm'%(oddp_tag, qid, mu, fbdoc, svddim)
        elif run_tag == '.rf' or run_tag == '.rf.oddp':
            ifilename = '../qtfs%s/q%d.okw.desc.fbdocs%d.svddim%d.rf.svm'%(oddp_tag, qid, fbdoc, svddim)
        elif run_tag == '.resample' or run_tag == '.resample.oddp':
            ifilename = '../qtfs_linear%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.nsample.%d.svm'%(oddp_tag, qid, mu, fbdoc, svddim, nsample)
        elif run_tag == '.reweighted':
            ifilename = '../qtfs_linear%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.reweighted.svm'%(oddp_tag, qid, mu, fbdoc, svddim)
        
        # query term feature file not exist
        if not os.path.exists(ifilename):
            continue
            #os.system('./query_term_feature_gen %d ../res/t%d.okw.mu%d.res %d %d %s'%(qid, trecid, mu, fbdoc, svddim, ifilename))
        
        ifile=open(ifilename,'r')
        for line in ifile:
            ofile.write('%s\n'%line.strip())

        ifile.close()

    ofile.close()
    return ofilename

def train_svm(train_qids, fbdoc, svddim, run_tag, nsample=None):
    oddp_tag = '_oddp' if '.oddp' in run_tag else ''
    prob_tag = ' -b 1 ' if '.oddp' not in run_tag else ' '
    nsample_tag = '.nsample.%d' % nsample if not nsample == None else ''
    train_qids.sort()
    svmname = None
    if train_qids[-1]-train_qids[0]+1 != len(train_qids):
        for index in xrange(len(train_qids)):
            if train_qids[index]-train_qids[0]!=index:
                svmname = 'q%d-%d-%d.%d.desc.fbdoc%d.svddim%d%s%s'%(train_qids[0],train_qids[index],train_qids[-1], len(train_qids),fbdoc,svddim,run_tag,nsample_tag)
                break
    else:
        svmname = 'q%d-%d.%d.desc.fbdoc%d.svddim%d%s%s'%(train_qids[0],train_qids[-1], len(train_qids),fbdoc,svddim,run_tag, nsample_tag)
    
    # read training examples
    ofilename = '../tmp/%s.train.svm'%svmname
    ofile=open(ofilename,'w')
    for qid in train_qids:
        trecid = qid2trecid(qid)
        mu = mus[trecid]
        ifilename = ''
        if run_tag == '' or run_tag == '.oddp':
            ifilename = '../qtfs%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.svm'%(oddp_tag, qid, mu, fbdoc, svddim)
        elif run_tag == '.rf' or run_tag == '.rf.oddp':
            ifilename = '../qtfs%s/q%d.okw.desc.fbdocs%d.svddim%d.rf.svm'%(oddp_tag, qid, fbdoc, svddim)
        elif run_tag == '.resample' or run_tag == '.resample.oddp':
            ifilename = '../qtfs_linear%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.nsample.%d.svm'%(oddp_tag, qid, mu, fbdoc, svddim, nsample)
        elif run_tag == '.reweighted':
            ifilename = '../qtfs_linear%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.reweighted.svm'%(oddp_tag, qid, mu, fbdoc, svddim)
        
        # query term feature file not exist

        if not os.path.exists(ifilename):
            continue
            #os.system('./query_term_feature_gen %d ../res/t%d.okw.mu%d.res %d %d %s'%(qid, trecid, mu, fbdoc, svddim, ifilename))
        
        ifile=open(ifilename,'r')
        for line in ifile:
            ofile.write('%s\n'%line.strip())

        ifile.close()

    ofile.close()

    #rescale train files
    os.system('../libsvm-3.14/svm-scale -l -u -s ../tmp/%s.rescale.param %s > %s.rescale' % (svmname, ofilename, ofilename))

    #train svm on condor
    jobstr = '%s.%s'%(svmname, strftime('%Y%m%d'))
    jobname = '.jobs/' + jobstr + '.job'
    f = open(jobname, 'w')
    f.write('Universe = vanilla\n')
    f.write('initialdir = /bos/usr2/gzheng/TermRecall/src\n')
    f.write('executable = /opt/python27/bin/python\n')
    f.write('arguments = ../libsvm-3.14/tools/gridregression.py -out ../out/%s.out %s %s.rescale\n'%(svmname, prob_tag, ofilename))
#    f.write('arguments = cv_regression.py %s.rescale ../out_L1/%s.out\n'%(ofilename, svmname))
    f.write('output = .jobs/'+jobstr+'.out\n')
    f.write('log = .jobs/'+jobstr+'.log\n')
    f.write('error = .jobs/'+jobstr+'.err\n')
    f.write('queue\n')
    f.close()
    os.system('condor_submit '+ jobname)    


def le_train_svmlight(train_qids, dev_qids, test_qids, fbdoc, svddim, run_tag):
    train_file = prepare_svm_file(train_qids, fbdoc, svddim, run_tag, 'train')
    train_prefix = train_file.split('/')[-1].split('.fbdoc')[0]
    dev_file   = prepare_svm_file(dev_qids,   fbdoc, svddim, run_tag, 'dev', '.' +train_prefix)
    test_file  = prepare_svm_file(test_qids,  fbdoc, svddim, run_tag, 'test','.' +train_prefix)
    
    # rescale train files
    os.system('../libsvm-3.14/svm-scale -l -u -s %s.rescale.param %s > %s.rescale' % (train_file, train_file, train_file))

    # rescale dev and test file w.r.t train files
    os.system('../libsvm-3.14/svm-scale -r %s.rescale.param %s > %s.rescale' % (train_file, dev_file, dev_file))
    os.system('../libsvm-3.14/svm-scale -r %s.rescale.param %s > %s.rescale' % (train_file, test_file, test_file))

    jobstr = '%d.%d.%d.%d.%d.%d.%d.%d.%s' %(train_qids[0], train_qids[-1], dev_qids[0], dev_qids[-1], test_qids[0], test_qids[-1], fbdoc, svddim, strftime('%Y%m%d'))
    jobname = '.jobs/' + jobstr + '.job'
    f = open(jobname, 'w')
    f.write('Universe = vanilla\n')
    f.write('initialdir = /bos/usr2/gzheng/TermRecall/src\n')
    f.write('executable = /opt/python27/bin/python\n')
    f.write('arguments = ./le_train_svm_diss.py %s %s %s %d\n'%(train_file, dev_file, test_file, test_qids[0]))
    f.write('output = .jobs/'+jobstr+'.out\n')
    f.write('log = .jobs/'+jobstr+'.log\n')
    f.write('error = .jobs/'+jobstr+'.err\n')
    f.write('queue\n')
    f.close()
    os.system('condor_submit '+ jobname)    
    
    '''
    # CONDOR PART BEGINS    
    # train svmlight
    os.system('./svm_learn -v 0 -z r -t 2 -g %d %s.rescale %s.model' % (gamma, train_file, train_file))
    
    # test on dev used to do CV
    # we can also do test on testing so that we do not to do again after we have chose the right params
    os.system('./svm_classify -v 0 %s.rescale %s.model %s.out' % ( dev_file, train_file, dev_file))
    os.system('./svm_classify -v 0 %s.rescale %s.model %s.out' % ( test_file, train_file, test_file))

    #generate weighted query
    oddp_tag = '_oddp' if '.oddp' in run_tag else ''
    prob_tag = ' -b 1 ' if '.oddp' not in run_tag else ' '

    # oddp
    if prob_tag == ' ':
        os.system('./wqg_oddP %s %s.out %s.q'% (test_file, test_file, test_file))
        os.system('./wqg_oddP %s %s.out %s.q'% (dev_file, dev_file, dev_file))
    # p
    else:
        os.system('./wqg_P %s %s.out %s.q' % (test_file, test_file, test_file))
        os.system('./wqg_P %s %s.out %s.q' % (dev_file, dev_file, dev_file))

    #NOTE HERE:
    #Test qids should be in the same trec dataset!!!
    trecid = qid2trecid(test_qids[0])

    #search the query!
    os.system('/bos/usr2/gzheng/.local/bin/IndriRunQuery /bos/tmp4/lezhao/gov2/queries/stoplist.param -count=1000 %s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:d,mu:%d > ../res/%s.res' %(dev_file, trecid, mus[trecid], dev_file.split('/')[-1]))
    os.system('/bos/usr2/gzheng/.local/bin/IndriRunQuery /bos/tmp4/lezhao/gov2/queries/stoplist.param -count=1000 %s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:d,mu:%d > ../res/%s.res' %(test_file, trecid, mus[trecid], test_file.split('/')[-1]))
    
    #evaluate the results!
    eval_output_dev = subprocess.check_output(('./trec_eval ../qrels/qrels.t%d.txt ../res/%s.res' %(trecid, dev_file.split('/')[-1])).split(' '))
    eval_output_test = subprocess.check_output(('./trec_eval ../qrels/qrels.t%d.txt ../res/%s.res' %(trecid, test_file.split('/')[-1])).split(' '))

    eval_map_dev = float(eval_output_dev.strip().split('\n')[5].split('\t')[-1])
    eval_map_test = float(eval_output_test.strip().split('\n')[5].split('\t')[-1])
    return (train_qids[0], train_qids[-1], dev_qids[0], dev_qids[-1], eval_map_dev, eval_map_test, fbdoc, svddim ,gamma)

    # CONDOR PART ENDS

    '''

def best_svm_performance(train_qids, test_qids, run_tag=''):
    trecno = qid2trecid(train_qids[0])
    if run_tag == '.resample' or run_tag == '.resample.oddp':
        for nsample in range(1,31):
            train_svm(train_qids, good_fbdocs[trecno], good_svddims[trecno], run_tag, nsample)
    else:
        for fbdoc in ([500] if run_tag == '.rf' or run_tag == '.rf.oddp' else fbdocs):
            #    for fbdoc in range(10,101,10):
            for svddim in svddims:
            #(svddims_resample if run_tag == '.resample' or run_tag == '.resample.oddp' else svddims):
                train_svm(train_qids, fbdoc, svddim, run_tag)
                
                        
def best_svm_map_performance(train_qids, test_qids, run_tag=''):
    K = 5
#    for k in range(2,K): # k fold cv
    for k in range(1,K+1): # k fold cv
        print 'Fold %d ...' % k
        res_list = []
        dev_qids_new = []
        test_qids_new = []
        size = len(test_qids) / K

        for i in range(k*size-size,k*size):
            test_qids_new.append(test_qids[i])
        for i in range(k*size, len(test_qids)):
            dev_qids_new.append(test_qids[i])
        for i in range(0, k*size-size):
            dev_qids_new.append(test_qids[i])

        trecid=qid2trecid(test_qids[0])
        
        #for fbdoc in fbdocs:
        mu = 900
        if trecid == 9 or trecid ==10:
            mu = 1500

        for fbdoc in getFbdocs(trecid, mu):
            for svddim in [150]:
            #for svddim in svddims:
                # run on condor
                '''
                jobstr = '%d.%d.%d.%d.%d.%d.%s' %(train_qids[0], train_qids[-1], dev_qids_new[0], dev_qids_new[-1], fbdoc, svddim, strftime('%Y%m%d'))
                jobname = '.jobs/' + jobstr + '.job'
                f = open(jobname, 'w')
                f.write('Universe = vanilla\n')
                f.write('initialdir = /bos/usr2/gzheng/TermRecall/src\n')
                f.write('executable = /opt/python27/bin/python\n')
                f.write('arguments = ./le_train_svm.py -out ../out/%s.out %s %s.rescale\n'%(svmname, prob_tag, ofilename))
    #    f.write('arguments = cv_regression.py %s.rescale ../out_L1/%s.out\n'%(ofilename, svmname))
                f.write('output = .jobs/'+jobstr+'.out\n')
                f.write('log = .jobs/'+jobstr+'.log\n')
                f.write('error = .jobs/'+jobstr+'.err\n')
                f.write('queue\n')
                f.close()
                os.system('condor_submit '+ jobname)    
                '''
                
                le_train_svmlight(train_qids, dev_qids_new, test_qids_new, fbdoc, svddim, run_tag)
                '''
                for gamma in gammas:
                    res_list.append(le_train_svmlight(train_qids, dev_qids_new, test_qids_new, fbdoc, svddim, gamma, run_tag))

        res_list.sort(reverse=True)
        print res_list[0]
        '''


def call_main(argstr):
    args = argstr.split(' ')
    train_qids = []
    test_qids = []
    run_tag = ''
    
    for arg in args:
        if arg.startswith("TRAIN"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            train_qids += range(start_qid, end_qid+1)
        elif arg.startswith("TEST"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            test_qids += range(start_qid, end_qid+1)
        elif arg == 'ODDP':
            run_tag = '.oddp'
        elif arg == 'RF':
            run_tag = '.rf'
        elif arg == 'REWEIGHTED':
            run_tag = '.reweighted'
        elif arg == 'RESAMPLE':
            run_tag = '.resample'
        elif arg == 'RF.ODDP':
            run_tag = '.rf.oddp'
        elif arg == 'RESAMPLE.ODDP':
            run_tag = '.resample.oddp'
        else:
            print 'Train/Test dataset error!'
            exit(1)

    if False:
        best_svm_performance(train_qids, test_qids, run_tag)
    else:
        best_svm_map_performance(train_qids, test_qids, run_tag)

if __name__ == '__main__':
    if(len(sys.argv)<3):
        print 'Usage: %s TRAIN:<train_qids_range> TEST:<test_qids_range>' % sys.argv[0]
        print 'Sample: %s TRAIN:151-200 TEST:201-250'%sys.argv[0]
        exit(1)
        
    train_qids = []
    test_qids = []
    
    for arg in sys.argv[1:]:
        if arg.startswith("TRAIN"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            train_qids += range(start_qid, end_qid+1)
        elif arg.startswith("TEST"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            test_qids += range(start_qid, end_qid+1)
        else:
            print 'Train/Test dataset error!'
            exit(1)

    best_svm_performance(train_qids, test_qids)
