/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include <cassert>
#include <queue>
#include <vector>
#include <string>
#include "leveldb/db.h"
#include "leveldb/write_batch.h"
#include "zlib.h"
#include <iostream>
using namespace std;

int main(int argc, char** argv)
{
    if(argc != 3)
    {
        cout<<"Usage: "<<argv[0]<<" <leveldb_path> <ngram.gz>"<<endl;
        exit(0);
    }
    

    // Create leveldb
    leveldb::DB* db;
    leveldb::Options options;
    options.create_if_missing = true;
    leveldb::Status status = leveldb::DB::Open(options, argv[1], &db);
    assert(status.ok());

    // Write records into it
    unsigned char unzippedBuffer[8192];
    unsigned int unzippedBytes;
    string key_buffer = "";
    string value_buffer = "";
    string key = "";
    string value = "";

    gzFile infile = gzopen(argv[2], "rb");

    if (infile == NULL)
    {
        cout<<"Error: Failed to open file!"<<endl;
        exit(1);
    }

    long long count = 0;

    while(true)
    {
        unzippedBytes = gzread(infile, unzippedBuffer, 8192);
        if(unzippedBytes > 0)
        {
            for(int i=0;i<unzippedBytes;i++)
            {
                if(unzippedBuffer[i] == '\t')
                {
                    key = key_buffer;
                    key_buffer = "";
                    value_buffer = "";
                }
                else if(unzippedBuffer[i] == '\n')
                {
                    value = value_buffer;
                    value_buffer = "";
                    key_buffer = "";
                    db->Put(leveldb::WriteOptions(), key, value);
                    count ++;
                    if(count % 1000 == 0)
                        cout<<count<<" records inserted."<<endl;
                }
                else
                {
                    key_buffer += unzippedBuffer[i];
                    value_buffer += unzippedBuffer[i];
                }
            }

        }
        else
            break;
    }

    gzclose(infile);

    delete db;
}
