# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

from Constants import *
import subprocess
import os
import glob
import sys

def trecid2qids(trecid):
    if trecid>=13:
        trecid+=1
    
    return range(trecid*50+1,trecid*50+51)

def qid2trecid(qid):
    if qid>700:
        qid-=50

    return (qid-1)/50

def cv_svm(train_qids, test_qids, k):
    for fbdoc in fbdocs:
        for svddim in svddims:
            train_svm(train_qids, fbdoc, svddim)
        

def train_svm(train_qids, fbdoc, svddim):
    print 'Training fbdoc=%d svddim=%d' % (fbdoc, svddim)
    train_qids.sort()
    svmname = None
    if train_qids[-1]-train_qids[0]+1 != len(train_qids):
        for index in xrange(len(train_qids)):
            if train_qids[index]-train_qids[0]!=index:
                svmname = 'q%d-%d-%d.%d.desc.fbdoc%d.svddim%d.oddp'%(train_qids[0],train_qids[index],train_qids[-1], len(train_qids),fbdoc,svddim)
                break
    else:
        svmname = 'q%d-%d.%d.desc.fbdoc%d.svddim%d.oddp'%(train_qids[0],train_qids[-1], len(train_qids),fbdoc,svddim)
    
    # read training examples
    ofilename = '../tmp/%s.train.svm'%svmname
    ofile=open(ofilename,'w')
    for qid in train_qids:
        trecid = qid2trecid(qid)
        mu = mus[trecid]
        ifilename = '../qtfs_oddp/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.svm'%(qid, mu, fbdoc, svddim)
        # query term feature file not exist
#        if not os.path.exists(ifilename):
 #           os.system('./query_term_feature_gen %d ../res/t%d.okw.mu%d.res %d %d %s'%(qid, trecid, mu, fbdoc, svddim, ifilename))
        
        ifile=open(ifilename,'r')
        for line in ifile:
            ofile.write('%s\n'%line.strip())

        ifile.close()

    ofile.close()

    #rescale train files
    os.system('../libsvm-3.14/svm-scale -l -u -s ../tmp/%s.rescale.param %s > %s.rescale' % (svmname, ofilename, ofilename))

    #train svm
    grid_output = subprocess.check_output(('/opt/python27/bin/python ../libsvm-3.14/tools/gridregression.py -out ../out/%s.out %s.rescale'%(svmname, ofilename)).split(' '))
    
    results = grid_output.strip().split('\n')[-1].split(' ')
    c = float(results[1])
    g = float(results[2])
    p = float(results[3])
    mse = float(results[4])
    return (mse, (fbdoc, svddim, c, g, p))

def best_svm_performance(train_qids, test_qids):
    results = []
    for fbdoc in fbdocs:
        for svddim in svddims:
            results.append(train_svm(train_qids, fbdoc, svddim))

    fbdoc = None
    svddim = None
    
    #Sort the results, pop the params which leads to the lease mse
    #Feature file for the best results shuld exist
    results.sort()
    print results[0]
    best_fbdoc = results[0][1][0]
    best_svddim = results[0][1][1]
    best_c = results[0][1][2]
    best_g = results[0][1][3]
    best_p = results[0][1][4]
    
    # training examples should be already
    # only need to get testing examples
    train_qids.sort()
    svmname = None
    if train_qids[-1]-train_qids[0]+1 != len(train_qids):
        for index in xrange(len(train_qids)):
            if train_qids[index]-train_qids[0]!=index:
                svmname = 'q%d-%d-%d.%d.desc.fbdoc%d.svddim%d.oddp'%(train_qids[0],train_qids[index],train_qids[-1], len(train_qids),best_fbdoc,best_svddim)
                break
    else:
        svmname = 'q%d-%d.%d.desc.fbdoc%d.svddim%d.oddp'%(train_qids[0],train_qids[-1], len(train_qids),best_fbdoc,best_svddim)

    ofilename = '../tmp/%s.test.svm'%svmname
    ofile=open(ofilename,'w')
    for qid in test_qids:
        trecid = qid2trecid(qid)
        mu = mus[trecid]
        ifilename = '../qtfs_oddp/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.svm'%(qid, mu, best_fbdoc, best_svddim)
        # query term feature file not exist
#        if not os.path.exists(ifilename):
#            os.system('./query_term_feature_gen %d ../res/t%d.okw.mu%d.res %d %d %s'%(qid, trecid, mu, best_fbdoc, best_svddim, ifilename))

        ifile=open(ifilename,'r')
        for line in ifile:
            ofile.write('%s\n'%line.strip())

        ifile.close()

    ofile.close()

    #rescale test files
    os.system('../libsvm-3.14/svm-scale -r ../tmp/%s.rescale.param %s > %s.rescale' % (svmname, ofilename, ofilename))

    #train final svm
    os.system('../libsvm-3.14/svm-train -s 3 -c %f -g %f -p %f ../tmp/%s.train.svm.rescale ../tmp/%s.model' % (best_c, best_g, best_p, svmname, svmname))

    #test it!
    os.system('../libsvm-3.14/svm-predict ../tmp/%s.test.svm.rescale ../tmp/%s.model ../tmp/%s.out' % (svmname, svmname, svmname))
    
    #generate weighted query
    os.system('./wqg_oddP ../tmp/%s.test.svm ../tmp/%s.out ../tmp/%s.q'% (svmname, svmname, svmname))

    #NOTE HERE:
    #Test qids should be in the same trec dataset!!!
    trecid = qid2trecid(test_qids[0])

    #search the query!
    os.system('/bos/usr2/gzheng/.local/bin/IndriRunQuery /bos/tmp4/lezhao/gov2/queries/stoplist.param -count=1000 ../tmp/%s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:d,mu:%d > ../res/%s.res' %(svmname, trecid, mus[trecid], svmname))
    
    #evaluate the results!
    eval_output = subprocess.check_output(('./trec_eval ../qrels/qrels.t%d.txt ../res/%s.res' %(trecid, svmname)).split(' '))

    eval_map = eval_output.strip().split('\n')[5]
    return float(eval_map.split('\t')[-1])
    
if __name__ == '__main__':
    if(len(sys.argv)<3):
        print 'Usage: %s TRAIN:<train_qids_range> TEST:<test_qids_range>' % sys.argv[0]
        print 'Sample: %s TRAIN:151-200 TEST:201-250'%sys.argv[0]
        exit(1)
        
    train_qids = []
    test_qids = []
    
    for arg in sys.argv[1:]:
        if arg.startswith("TRAIN"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            train_qids += range(start_qid, end_qid+1)
        elif arg.startswith("TEST"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            test_qids += range(start_qid, end_qid+1)
        else:
            print 'Train/Test dataset error!'
            exit(1)

    print best_svm_performance(train_qids, test_qids)
