# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

import sys
import glob
import os
import subprocess

if __name__ == '__main__':
    eval_dirs_to_do = glob.glob('../tmp_20140607_stopped_sd_CIKM-*_mix')
    
    robust04_data_file = open('cikm.robust04.dat', 'w')
    wt10g_data_file = open('cikm.wt10g.dat', 'w')
    gov2_data_file = open('cikm.gov2.dat', 'w')
    clueweb09b_data_file = open('cikm.clueweb09b.dat', 'w')

    data_file_dict = dict()
    data_file_dict['robust04'] = robust04_data_file
    data_file_dict['wt10g'] = wt10g_data_file
    data_file_dict['gov2'] = gov2_data_file
    data_file_dict['clueweb09b'] = clueweb09b_data_file

    robust04_legend_file = open('cikm.robust04.legend', 'w')
    wt10g_legend_file = open('cikm.wt10g.legend', 'w')
    gov2_legend_file = open('cikm.gov2.legend', 'w')
    clueweb09b_legend_file = open('cikm.clueweb09b.legend', 'w')
    
    legend_file_dict = dict()
    legend_file_dict['robust04'] = robust04_legend_file
    legend_file_dict['wt10g'] = wt10g_legend_file
    legend_file_dict['gov2'] = gov2_legend_file
    legend_file_dict['clueweb09b'] = clueweb09b_legend_file

    for collection_str in ['robust04', 'wt10g', 'gov2', 'clueweb09b']:
        lm_eval = 'lm.%s.eval' % collection_str
        sd_eval = 'sd.%s.eval' % collection_str
        cmd = '/opt/python27/bin/python cikm_robust.py %s %s' % (lm_eval, sd_eval)
        output = subprocess.check_output(cmd.split(' ')).strip()
        data_file_dict[collection_str].write('%s\n' % output)
        legend_file_dict[collection_str].write('SD\n')
    
    for eval_dir in eval_dirs_to_do:
        for filename in glob.glob('%s/*.eval' % eval_dir):
            collection_str = filename.split('/')[-1].split('.')[0]
            print filename, collection_str
            lm_eval = 'lm.%s.eval' % collection_str
            cmd = '/opt/python27/bin/python cikm_robust.py %s %s' % (lm_eval, filename)
            output = subprocess.check_output(cmd.split(' ')).strip()

            data_file_dict[collection_str].write('%s\n'% output)
            legend_file_dict[collection_str].write('DeepTR-SD (%s)\n' % filename.split('/')[-2].split('-')[-1].split('_')[0])

    for collection_str in ['robust04', 'wt10g', 'gov2', 'clueweb09b']:
        data_file_dict[collection_str].close()
        legend_file_dict[collection_str].close()
        
        mtb_cmd = '/opt/matlab/7.13/bin/matlab   -nodesktop -nosplash  -r \"plot_fig(\'%s\', \'%s\')\"' % ('cikm.%s.dat' % collection_str, 'cikm.%s.legend' % collection_str)
        os.system(mtb_cmd)


