# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

import os
import sys
from query_term_feature_le import *
from time import strftime

def prepare_svm_file(qtfs_folder, qids, mu, fbdoc, svddim, filename):
    with open(filename, 'w') as svm_out:
        for qid in qids:
            ifilename = '%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.svm.true.recall' %(qtfs_folder, qid, mu, fbdoc, svddim)
            if not os.path.exists(ifilename):
                continue

            with open(ifilename, 'r') as ifile:
                for line in ifile:
                    svm_out.write('%s\n' % line.strip())


def tid2range(tid):
    if tid==21:
        return range(2001, 2100)
    if tid==22:
        return range(2101, 2201)
    
    if tid==35:
        return range(151, 301)
    if tid==37:
        return range(151, 401)
    if tid==39:
        return range(151, 501)

    if tid==13 or tid==14:
        tid+=1

    return range(tid*50+1, tid*50 + 51)

def tid2range2(tid):
    if tid==7:
        return range(351, 431) + range(441,451)
    if tid==8: 
        return range(431, 441)

def get_test_tid(tid):
    if tid==35:
        return 6
    if tid==37:
        return 8
    if tid==39:
        return 10
    if tid==8:
        return 6
    else:
        return tid+1

def get_test_tid2(tid):
    if tid==35:
        return 6
    if tid==37:
        return 8
    if tid==39:
        return 10
    else:
        return tid

def getTTFbdocs(test_tid):
    if test_tid ==3:
        return [300]
    elif test_tid==7:
        return [150]
    elif test_tid==9:
        return [250]
    elif test_tid==11:
        return [600]
    elif test_tid==35:
        return [300]
    elif test_tid==37:
        return [150]
    elif test_tid==39:
        return [250]


if __name__ == '__main__':
    if len(sys.argv)!= 5:
        print 'Usage: %s <qtfs_folder> <tmp_foler> <QL|SD> <LM|BM25>' % sys.argv[0]
        exit(0)

        
    qtfs_folder = sys.argv[1]
    tmp_folder = sys.argv[2]
    query_model = sys.argv[3]
    retrieval_model = sys.argv[4]

    if (query_model!= 'QL' and query_model != 'SD') or (retrieval_model != 'LM' and retrieval_model !='BM25'):
        print 'Usage: %s <qtfs_folder> <tmp_foler> <QL|SD> <LM|BM25>' % sys.argv[0]
        exit(0)

    #for trecno in range(3,13,2) + [35, 37,39]:
    #for trecno in range(7,13,2):
    #for trecno in [13]:
    #for trecno in range(7,15,2) + [21]:
    for trecno in [7, 9, 13, 21]: # trec7, wt10g, gov2, clueweb09
    #for trecno in range(7,13,2):
    #for trecno in range(7,15,2):
    #for trecno in range(13,15,2):
    #for trecno in [21]:
    #for trecno in range(3,7,2):
    #for trecno in range(7,9,2):
    #for trecno in range(4,13,2):
    #for trecno in [5,7,9]:
    #for trecno in [3]:
        test_tid = get_test_tid(trecno)
        train_qids = tid2range(trecno)
        test_qids = tid2range(test_tid)
        
        mu = 2500
        
        #    for fbdoc in [300]:
        #for fbdoc in [100]:
        for fbdoc in [600]:
            
            #    for fbdoc in getFbdocs(test_tid, mu):
            #    for fbdoc in getTTFbdocs(trecno):
            for svddim in [50]:#,100,50]:
                #            tmp_folder = '../tmp_20140123_all'
                #            tmp_folder = '../tmp_20140123_FG2'
                #            tmp_folder = '../tmp_20140123_FG3'
                #            tmp_folder = '../tmp_20140123_FG4'
                #            tmp_folder = '../tmp_20140123_FG5'
                #            tmp_folder = '../tmp_20140123_FG6'
                
                train_svm_filename = '%s/q%d-%d.%d.desc.fbdoc%d.svddim%d.train.svm' % (tmp_folder, train_qids[0], train_qids[-1], len(train_qids), fbdoc, svddim)
                test_svm_filename = '%s/q%d-%d.%d.desc.fbdoc%d.svddim%d.train.svm' % (tmp_folder, test_qids[0], test_qids[-1], len(test_qids), fbdoc, svddim)
                prepare_svm_file(qtfs_folder, train_qids, mu, fbdoc, svddim, train_svm_filename);
                prepare_svm_file(qtfs_folder, test_qids, mu, fbdoc, svddim, test_svm_filename);
                
                os.system('/opt/python27/bin/python2.7 svm2mtb_complete.py %s' % (train_svm_filename))
                os.system('/opt/python27/bin/python2.7 svm2mtb_complete.py %s' % (test_svm_filename))
                
                jobstr = '%s/%s.%s' %(tmp_folder, train_svm_filename.split('/')[-1], strftime('%Y%m%d'))
                jobname = '%s.job' % jobstr
                f = open(jobname, 'w')
                f.write('Universe = vanilla\n')
                f.write('initialdir = /bos/usr0/gzheng/TermRecall/src\n')
                f.write('executable = /opt/python27/bin/python\n')
                f.write('arguments = deep_lasso_condor.py %s %s %d %s %s\n'%( train_svm_filename, test_svm_filename, test_tid, query_model, retrieval_model))
                f.write('output = '+jobstr+'.out\n')
                f.write('log = '+jobstr+'.log\n')
                f.write('error = '+jobstr+'.err\n')
                f.write('queue\n')
                f.close()
                os.system('condor_submit '+ jobname)
