# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

import os
from time import strftime
from doit_deep_cikm import get_all_qids

def prepare_wsd_file(qids, filename):
    with open(filename, 'w') as wsd_out:
        for qid in qids:
            ifilename = '../qtfs_wsd/q%d.wsd' % qid
            if not os.path.exists(ifilename):
                continue

            with open(ifilename, 'r') as ifile:
                for line in ifile:
                    wsd_out.write('%s\n' % line.strip())

def tid2range(tid):
    if tid==21:
        return range(2001, 2100)
    if tid==22:
        return range(2101, 2201)
    
    if tid==35:
        return range(151, 301)
    if tid==37:
        return range(151, 401)
    if tid==39:
        return range(151, 501)

    if tid==13 or tid==14:
        tid+=1

    return range(tid*50+1, tid*50 + 51)

def get_test_tid(tid):
    if tid==35:
        return 6
    if tid==37:
        return 8
    if tid==39:
        return 10
    if tid==8:
        return 6
    else:
        return tid+1

def get_test_tid2(tid):
    if tid==35:
        return 6
    if tid==37:
        return 8
    if tid==39:
        return 10
    else:
        return tid

if __name__ == '__main__':
    #for trecno in range(3,13,2) + [35, 37,39]:
    #for trecno in range(3,13,2):
#    for trecno in [7, 9, 13, 21]:
    for trecno in ['robust04', 'wt10g', 'gov2', 'clueweb09b']:
    #for trecno in range(3,7,2):
    #for trecno in range(7,9,2):
    #for trecno in range(4,13,2):
    #for trecno in [5,7,9]:
    #for trecno in [3]:
        for rmmodel in ['LM', 'BM25']:
#            test_tid = get_test_tid(trecno)
            train_qids = get_all_qids(trecno)
            test_qids = get_all_qids(trecno)
            
            tmp_folder = '../tmp_wsd'
            
            train_wsd_filename = '%s/q%d-%d.%d.desc.train.wsd' % (tmp_folder, train_qids[0], train_qids[-1], len(train_qids))
            test_wsd_filename  = '%s/q%d-%d.%d.desc.train.wsd' % (tmp_folder, test_qids[0], test_qids[-1], len(test_qids))
            prepare_wsd_file(train_qids, train_wsd_filename)
            prepare_wsd_file(test_qids,  test_wsd_filename)
            
            jobstr = 'WSD.%s.%s.%s' %(train_wsd_filename.split('/')[-1], rmmodel, strftime('%Y%m%d'))
            jobname = '.jobs/' + jobstr + '.job'
            f = open(jobname, 'w')
            f.write('Universe = vanilla\n')
            f.write('initialdir = /bos/usr0/gzheng/TermRecall/src\n')
            f.write('executable = /opt/python27/bin/python\n')
            f.write('arguments = wsd_condor.py %s %s %s %s\n'%(trecno, train_wsd_filename, test_wsd_filename, rmmodel))
            f.write('output = .jobs/'+jobstr+'.out\n')
            f.write('log = .jobs/'+jobstr+'.log\n')
            f.write('error = .jobs/'+jobstr+'.err\n')
            f.write('queue\n')
            f.close()
            os.system('condor_submit '+ jobname)    
            
