/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include "FeatureGenerator.h"
#include <indri/Index.hpp>
#include <vector>
#include <Eigen/Dense>
#include <sstream>

using namespace std;
using namespace indri::index;
using namespace indri::collection;
using namespace Eigen;

std::string str(int a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

std::string str(float a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}


int qid2trecid(int qid)
{
    if(qid>700)
    {
        qid -= 50;
    }

    return (qid-1)/50;
}
 
int main(int argc, char **argv)
{
    if(argc != 6)
    {
        cout<<"Usage: "<< argv[0]<<" <Query ID> <initial_res> <fbdocs> <svddim> <featurefle>"<<endl;
        exit(0);
    }

    int qid = atoi(argv[1]);
    int data_set = qid2trecid(qid);

    std::string initial_res(argv[2]);
    int fbdocs = atoi(argv[3]);
    int svddim = atoi(argv[4]);
    
    std::ofstream data_out(argv[5], std::ofstream::out);
    std::string idx = "/bos/tmp17/gzheng/trec/indext" + str(data_set) + "-v4.10";
    std::string qrel = "/bos/usr2/gzheng/Boolean/qrels/qrels.t" + str(data_set) + ".txt";

    if(qid==613)
    {
        //Query 613: What is it?
        //All stop words -> no retrieval results
        return 1;
    }

    cout<<"Processing query: "<<qid<<" fbdocs: "<<fbdocs<<" svddim: "<<svddim<<endl;
    std::string dep_res;
    // Trec 3 query dep
    if(data_set ==3 || data_set ==4)
        dep_res = "/bos/usr1/lezhao/Boolean/parses/t" + str(data_set) +"/" + str(qid) + ".dep";
    else
        dep_res = "/bos/usr1/lezhao/Boolean/parses/t" + str(data_set) +".desc/" + str(qid) + ".dep";
    
    // Seems now do not need to set query str
    std::string query_str = "";
    FeatureGenerator fg(idx, qid, query_str, initial_res);
    fg.generateTFIDFMatrix(fbdocs);
    std::string out_file(argv[5]);
    //fg.saveTFIDFDataMatrix(out_file);
    fg.compute_concept_matrix(svddim);

    // Compute PLSA features
    //fg.compute_concept_plsa_matrix(svddim);

    fg.load_qrel(qrel);
    fg.load_dep(dep_res);

    // Load stop words
    std::string stopwf("/bos/tmp4/lezhao/gov2/queries/stoplist.param");
    //fg.load_stop_words(stopwf);

    // Load le's list of outputed terms
    int begin_p = initial_res.find("mu") + 2;
    int end_p = initial_res.rfind(".res");
    std::string mu_str = initial_res.substr(begin_p, end_p - begin_p);
    int mu = atoi(mu_str.data());
        
    std::string fts_file;
    if(data_set <5)
    {
        if(mu==900)
            fts_file = "/bos/usr2/gzheng/Boolean/out/before5.nocla.svd3.fbdocs500.svddim150.oddP.mu900.svm";
        else
            fts_file = "/bos/usr2/gzheng/Boolean/out/before5.nocla.svd3.fbdocs300.svddim150.oddP.mu1500.svm";
    }
    if(data_set ==5)
    {
        if (mu==900)
            fts_file = "/bos/usr2/gzheng/Boolean/out/prs5.nocla.svd3.fbdocs500.svddim150.oddP.desc.mu900.svm";
        else
            fts_file = "/bos/usr2/gzheng/Boolean/out/prs5.nocla.svd3.fbdocs300.svddim150.oddP.desc.mu1500.svm";
    }
    else if(data_set < 8)
    {
        if(mu==900)
            fts_file = "/bos/usr2/gzheng/Boolean/out/before8.nocla.svd3.fbdocs200.svddim150.oddP.desc.mu900.svm";
        else
            fts_file = "/bos/usr2/gzheng/Boolean/out/before8.nocla.svd3.fbdocs300.svddim150.oddP.desc.mu1500.svm";
    }
    else if(data_set ==8)
    {
        // Set to 1500 for both 900 and 1500
        fts_file = "/bos/usr2/gzheng/Boolean/out/prs8.nocla.svd3.fbdocs300.svddim150.oddP.desc.mu1500.svm";
    }
    else if(data_set < 10)
    {
        if(mu==1500)
            fts_file = "/bos/usr2/gzheng/Boolean/out/before10.nocla.svd3.fbdocs300.svddim150.oddP.desc.mu1500.svm";
    }
    else if(data_set ==10)
    {
        fts_file = "/bos/usr2/gzheng/TermRecall/src/_for_t10.svm";
    }
    else if(data_set ==11)
    {
        fts_file = "/bos/usr2/gzheng/Boolean/out/prs11.nocla.svd3.fbdocs700.svddim150.oddP.desc.mu900.svm";
    }
    else if(data_set ==12)
    {
        fts_file = "/bos/usr2/gzheng/TermRecall/src/_for_t12.svm";
    } // Add for t13 and t14
    
    fg.load_output_terms(fts_file);

    // Generate 5 svm features
    data_out<<fg.generate_svm_features();

    // Generate 5 svm features and 9 tfidf features
    //data_out<<fg.generate_svm_beta_features();
    
    data_out.flush();
    data_out.close();
    
    return 0;
}


