/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include "FeatureGenerator.h"
#include <indri/Index.hpp>
#include <vector>
#include <Eigen/Dense>
#include <sstream>

using namespace std;
using namespace indri::index;
using namespace indri::collection;
using namespace Eigen;

std::string str(int a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

std::string str(float a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

int qid2trecid(int qid)
{
    if(qid>700)
    {
        qid -= 50;
    }

    return (qid-1)/50;
}
 
int main(int argc, char **argv)
{
    if(argc != 6)
    {
        cout<<"Usage: "<< argv[0]<<" <Query ID> <initial_res> <fbdocs> <svddim> <featurefle>"<<endl;
        exit(0);
    }

    int qid = atoi(argv[1]);
    int data_set = qid2trecid(qid);

    std::string initial_res(argv[2]);
    int fbdocs = atoi(argv[3]);
    int svddim = atoi(argv[4]);
    std::ofstream data_out(argv[5], std::ofstream::out);
    std::string idx = "/bos/tmp17/gzheng/trec/indext" + str(data_set) + "-v5.3";
    std::string qrel = "../qrels/qrels.t" + str(data_set) + ".txt";

    if(qid==613)
    {
        //Query 613: What is it?
        //All stop words -> no retrieval results
        return 1;
    }

    cout<<"Processing query: "<<qid<<" fbdocs: "<<fbdocs<<" svddim: "<<svddim<<endl;
    std::string dep_res = "../parses/t" + str(data_set) +".desc/" + str(qid) + ".dep";
    
    // Seems now do not need to set query str
    std::string query_str = "";
    FeatureGenerator fg(idx, qid, query_str, initial_res);
    fg.load_qrel(qrel);
    fg.generateRFMatrix(fbdocs);
    fg.compute_concept_matrix(svddim);
    fg.load_dep(dep_res);
    data_out<<fg.generate_svm_features();
    data_out.flush();
    data_out.close();
    
    return 0;
}
