/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include "FeatureGenerator.h"
#include <indri/Index.hpp>
#include <vector>
#include <Eigen/Dense>
#include <sstream>

using namespace std;
using namespace indri::index;
using namespace indri::collection;
using namespace Eigen;

std::string str(int a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

std::string str(float a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

int qid2trecid(int qid)
{
    if(qid>700)
    {
        qid -= 50;
    }

    return (qid-1)/50;
}
 
int main(int argc, char **argv)
{
    if(argc != 6)
    {
        cout<<"Usage: "<< argv[0]<<" <Query ID> <initial_res> <fbdocs> <svddim> <featurefle>"<<endl;
        exit(0);
    }

    int qid = atoi(argv[1]);
    int data_set = qid2trecid(qid);

    std::string initial_res(argv[2]);
    int fbdocs = atoi(argv[3]);
    int svddim = atoi(argv[4]);
    
    std::ofstream data_out(argv[5], std::ofstream::out);
    std::string idx = "/bos/tmp17/gzheng/trec/indext" + str(data_set) + "-v4.10";
    std::string qrel = "/bos/usr2/gzheng/Boolean/qrels/qrels.t" + str(data_set) + ".txt";

    if(qid==613)
    {
        //Query 613: What is it?
        //All stop words -> no retrieval results
        return 1;
    }

    cout<<"Processing query: "<<qid<<" fbdocs: "<<fbdocs<<" svddim: "<<svddim<<endl;
    std::string dep_res;
    // Trec 3 query dep
    if(data_set ==3 || data_set ==4)
        dep_res = "/bos/usr1/lezhao/Boolean/parses/t" + str(data_set) +"/" + str(qid) + ".dep";
    else
        dep_res = "/bos/usr1/lezhao/Boolean/parses/t" + str(data_set) +".desc/" + str(qid) + ".dep";
    
    // Seems now do not need to set query str
    std::string query_str = "";

    std::string query_part_file = "../queries/desc_parts_stopped/" + str(qid) + ".part";
    FeatureGenerator fg(idx, qid, query_str, query_part_file, initial_res, true);
    fg.generateTFIDFMatrixWithExpressions(fbdocs);
    std::string out_file(argv[5]);
    //fg.saveTFIDFDataMatrix(out_file);
    fg.compute_concept_matrix(svddim);

    // Compute PLSA features
    //fg.compute_concept_plsa_matrix(svddim);

    fg.load_qrel(qrel);

    // Load stop words
    //std::string stopwf("/bos/tmp4/lezhao/gov2/queries/stoplist.param");
    //fg.load_stop_words(stopwf);

    // Generate 5 svm features
    data_out<<fg.generate_svm_features_with_phrase();

    // Generate 5 svm features and 9 tfidf features
    //data_out<<fg.generate_svm_beta_features();
    
    data_out.flush();
    data_out.close();
    
    return 0;
}


