/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include "FeatureGenerator.h"
#include <indri/Index.hpp>
#include <vector>
#include <Eigen/Dense>
#include <sstream>

using namespace std;
using namespace indri::index;
using namespace indri::collection;
using namespace Eigen;

std::string str(int a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

std::string str(float a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}


int main(int argc, char **argv)
{
    if(argc != 9)
    {
        cout<<"Usage: "<< argv[0]<<" <Query ID> <initial_res> <idx> <qrel> <dep> <fbdocs> <svddim> <featurefle>"<<endl;
        exit(0);
    }
    

    int qid = atoi(argv[1]);
    std::string initial_res(argv[2]);
    std::string idx_path(argv[3]);
    std::string qrel_path(argv[4]);
    std::string dep_res(argv[5]);
    int fbdocs = atoi(argv[6]);
    int svddim = atoi(argv[7]);
    
    std::ofstream data_out(argv[8], std::ofstream::out);

    cout<<"Processing query: "<<qid<<" fbdocs: "<<fbdocs<<" svddim: "<<svddim<<endl;
    // Seems now do not need to set query str
    std::string query_str = "";
    FeatureGenerator fg(idx_path, qid, query_str, initial_res);
    fg.load_qrel(qrel_path);
    fg.generateRFMatrix(fbdocs);
    fg.compute_concept_matrix(svddim);
    fg.load_dep(dep_res);
    data_out<<fg.generate_svm_features();
    data_out.flush();
    data_out.close();
    
    return 0;
}


