# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

import os
import glob
import sys
from time import strftime

def getTrainQids(runid):
    if runid == 'st10':
        return range(1, 76)
    elif runid == 'st11':
        return range(1, 39)
    elif runid == 'st12':
        return range(1, 50)

def getTestQids(runid):
    if runid == 'st10':
        return range(76, 151)
    elif runid == 'st11':
        return range(39, 77)
    elif runid == 'st12':
        return range(50, 99)

def getQrel(runid):
    if runid == 'st10':
        return '../qrels/qrels.st10.1.txt'
    elif runid == 'st11':
        return '../qrels/qrels.st11.txt'
    elif runid == 'st12':
        return '../qrels/qrels.st12.txt'

if len(sys.argv) != 5:
    print "Usage: %s <runid> <fbdoc> <svddim> <gamma>" % sys.argv[0]
    print 'runid should be one of st10, st11 and st12'
    exit(1)

runid = sys.argv[1]
fbdoc = int(sys.argv[2])
svddim = int(sys.argv[3])
gamma = float(sys.argv[4])

train_qids = getTrainQids(runid)
test_qids = getTestQids(runid)

mu = 4000

num_res = {}
num_max = -1

qrel = getQrel(runid)

for qid in train_qids + test_qids:
    res_files = glob.glob('../clueweb09_res/%s/q%d.*mu%d*'%(runid,qid,mu))
    size = len(res_files)
    print qid, size
    num_res[qid] = size
    if size > num_max:
        num_max = size

exit(0)

#for i in range(0, num_max):
for i in [0]:
    # decide to use which feature file to construct the training file
    # 0 is for current query
    train_filename = '../sessiontrack_tmp/%s.q%d-%d.okw.mu%d.fbdoc%d.svddim%d.%d.train.svm' % (runid, train_qids[0], train_qids[-1], mu, fbdoc, svddim, i)
    test_filename = '../sessiontrack_tmp/%s.q%d-%d.okw.mu%d.fbdoc%d.svddim%d.%d.test.svm' % (runid, test_qids[0], test_qids[-1], mu, fbdoc, svddim, i)
    #test_filename = '../sessiontrack_tmp/%s.q%d-%d.okw.mu%d.fbdoc%d.svddim%d.0.test.svm' % (runid, test_qids[0], test_qids[-1], mu, fbdoc, svddim)

    train_out = open(train_filename, 'w')
    test_out = open(test_filename, 'w')

    for qid in train_qids + test_qids:
        refid = None
        if i<num_res[qid]:
            refid = i
        else:
            refid = num_res[qid] - 1

        
        infile_name = None
        if runid == 'st10':
            if refid == 0:
                infile_name = '../sessiontrack_qtfs/%s.q%d.2.2.okw.mu%d.fbdocs%d.svddim%d.svm'%(runid, qid, mu, fbdoc, svddim)
            else: # refid == 1
                infile_name = '../sessiontrack_qtfs/%s.q%d.2.1.okw.mu%d.fbdocs%d.svddim%d.svm'%(runid, qid, mu, fbdoc, svddim)        
        else: # st11 and st12
            if qid in train_qids:
                #infile_name = '../sessiontrack_qtfs/%s.q%d.%d.okw.mu%d.fbdocs%d.svddim%d.svm'%(runid, qid, refid, mu, fbdoc, svddim)
                infile_name = '../sessiontrack_qtfs/%s.q%d.0.okw.mu%d.fbdocs%d.svddim%d.svm'%(runid, qid, mu, fbdoc, svddim)
            else:
                #infile_name = '../sessiontrack_qtfs/%s.q%d.0.okw.mu%d.fbdocs%d.svddim%d.svm'%(runid, qid, mu, fbdoc, svddim)
                infile_name = '../sessiontrack_qtfs/%s.q%d.%d.okw.mu%d.fbdocs%d.svddim%d.svm'%(runid, qid, refid, mu, fbdoc, svddim)

        if os.path.exists(infile_name):
            infile = open(infile_name, 'r')
            for line in infile:
                if qid in train_qids:
                    train_out.write('%s\n'% line.strip())
                else:
                    test_out.write('%s\n'% line.strip())

            infile.close()

    train_out.close()
    test_out.close()

    jobstr = '%s.%s' %(train_filename.split('/')[-1], strftime('%Y%m%d'))
    print jobstr
    #continue
    jobname= '.jobs/' + jobstr + '.job'
    f = open(jobname, 'w')
    f.write('Universe = vanilla\n')
    f.write('initialdir = /bos/usr2/gzheng/TermRecall/src\n')
    f.write('executable = /opt/python27/bin/python\n')
    f.write('arguments = ./recall_train_predict.py %s %s %s %d %f\n'%(train_filename, test_filename, qrel, mu, gamma))
    f.write('output = .jobs/'+jobstr+'.out\n')
    f.write('log = .jobs/'+jobstr+'.log\n')
    f.write('error = .jobs/'+jobstr+'.err\n')
    f.write('queue\n')
    f.close()
    os.system('condor_submit '+ jobname)    

