/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <string>
#include <algorithm>

using namespace std; 

vector<string> split(const string &s, char delim) {
    vector<string> elems;
    stringstream ss(s);
    string item;
    while(getline(ss, item, delim)) {
        elems.push_back(item);
    }
    
    return elems;
}

class WeightedTerm
{
public:
    string term;
    double weight;
    
    WeightedTerm(string& term, double w)
    {
        this->term = term;
        this->weight = w;
    }
    ~WeightedTerm(){}

    bool operator<(const WeightedTerm& term2) const
    {
        return weight > term2.weight;
    }
};

float getWeight(int rank, int N)
{
    float sum = 0;
    for(int i=1;i<=N;i++)
        sum+= (1.0 / (float)i);

    return 1.0 /((float)rank)/sum;
}

int main(int argc, char **argv)
{
    if(argc != 3)
    {
        cout<<"Usage: "<< argv[0]<<" <SVM file> <result_output>"<<endl;
        exit(0);
    }

    
    ifstream svm_test(argv[1], ifstream::in);
    ofstream output(argv[2], ofstream::out);

    vector<WeightedTerm> terms;
    string line;
    double p_recall = 0;
    int pre_qid = -1;
    
    while(getline(svm_test, line))
    {
        stringstream ss(line);
        ss>>p_recall;
        //       svm_test>>p_recall;

        //cout<<p_recall<<endl;
        vector<string> splits = split(line, ' ');
        int size = splits.size();
        int qid = atoi((splits[size-1]).c_str());
        string term = splits[size-2];
        WeightedTerm wt(term, p_recall);
        if(pre_qid==-1 || qid == pre_qid)
            terms.push_back(wt);
        
        if(qid != pre_qid)
        {
            if(pre_qid != -1)
            {
                sort(terms.begin(), terms.end());

                for(int i=0;i<terms.size();i++)
                {
                    output<<pre_qid<<","<<terms[i].term<<','<<(i+1)<<','<<terms[i].weight<<endl;
                }
                

                terms.clear();
                WeightedTerm wt(term, p_recall);
                terms.push_back(wt);
            }
        }

        pre_qid = qid;        
    }

    // Output last
    if(pre_qid!=-1)
    {
        sort(terms.begin(), terms.end());
        
        for(int i=0;i<terms.size();i++)
        {
            output<<pre_qid<<","<<terms[i].term<<','<<(i+1)<<','<<terms[i].weight<<endl;
            //float weight = getWeight(i+1, terms.size());
            //            query_output<<weight<<" "<<terms[i].term<<" ";
            //query_output<<(weight+terms[i].weight)/2<<" "<<terms[i].term<<" ";
        }
        
        //query_output<<" )"<<endl;
        //query_output<<"\t</text></query>"<<endl;
        //query_output<<"</parameters>"<<endl;
    }

    output.close();
    svm_test.close();
    
    return 0;
}
