# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

import sys
import gensim
from collections import defaultdict

# dists is sorted 
def compute_features(dists_list):
    values = []

    # like term centrality
    values.append(dists_list[0][0])

    # like topic centrality
    sum = 0.0
    s = min(5, len(dists_list))
    for i in range(0, s):
        sum += dists_list[i][0]

    values.append(sum / s)

    # max dist
    max_dist = -1
    for item in dists_list:
        if max_dist < item[0]:
            max_dist = item[0]
            
    try:
        values.append(dists_list[s-1][0] / max_dist)
    except ZeroDivisionError:
        values.append(0)

    return values

if len(sys.argv)!=2:
    print 'Usage: %s <feature_svm_list>' % sys.argv[0]
    exit(0)

filenames= []
with open(sys.argv[1], 'r') as f:
    for line in f:
        filenames.append(line.strip())

# load model
print 'Model loading...'
model = gensim.models.Word2Vec.load_word2vec_format('word2vec/GoogleNews-vectors-negative300.bin', binary=True)
print 'Model loading done.'

for filename in filenames:
    terms = set()
    qid = None
    dists = defaultdict(list)
    with open(filename, 'r') as f:
        for line in f:
            parts = line.strip().split('\t')
            qid = int(parts[-1])
            term = parts[-2]
            if not term.startswith('#'):
                terms.add(term)

    for term in terms:
        for another_term in terms:
            if term!=another_term:
                try:
                    sim = model.similarity(term, another_term)
                    print >> sys.stderr, 'KEY ERROR FOR :', filename
                except KeyError:
                    sim = 0
                dists[term].append((sim, another_term))

    for term in dists:
        dists[term] = sorted(dists[term], reverse=True)

        
    filename_last_part = filename.split('/')[-1]
    print filename
    with open(filename, 'r') as f, open('../qtfs_20140429_stopped_sd_raw_SIGIR2_external_data_extended_inqsum_deep/%s' % filename_last_part, 'w') as w:
        for line in f:
            parts = line.strip().split('\t')
            qid = int(parts[-1])
            term = parts[-2]
            if not term.startswith('#'):
                feats = compute_features(dists[term]);
                feature_parts = line.strip().split('#', 1)
                w.write('%s 1001:%f 1002:%f 1003:%f #%s\n' % (feature_parts[0], feats[0], feats[1], feats[2], feature_parts[1]))



                
            
    


