# ==========================================================================
#  Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
# 
#  Use of the Lemur Toolkit for Language Modeling and Information Retrieval
#  is subject to the terms of the software license set forth in the LICENSE
#  file included with this software (and below), and also available at
#  http://www.lemurproject.org/license.html
# 
# ==========================================================================

import sys
import gensim
from collections import defaultdict

def compute_rep_feat_mean(rep_list, mean_list):
    feats = []
    for i in range(0, len(rep_list)):
        feats.append(rep_list[i] - mean_list[i]);
            
    return feats

if len(sys.argv)!=4:
    print 'Usage: %s <feature_svm_list> <model.bin> <output_dir>' % sys.argv[0]
    exit(0)

model_file = sys.argv[2]
output_dir = sys.argv[3]

filenames= []
with open(sys.argv[1], 'r') as f:
    for line in f:
        filenames.append(line.strip())

# load model
print 'Loading model %s ...' % model_file
#model = gensim.models.Word2Vec.load_word2vec_format('word2vec/GoogleNews-vectors-negative300.bin', binary=True)
model = gensim.models.Word2Vec.load_word2vec_format('%s'% model_file, binary=True)
print 'Loading model done.'

for filename in filenames:
    terms = set()
    qid = None
    reps = defaultdict(list)
    
    with open(filename, 'r') as f:
        for line in f:
            parts = line.strip().split('\t')
            qid = int(parts[-1])
            term = parts[-2]
            if not term.startswith('#'):
                terms.add(term)

    mean_rep = []
    for i in range(0, 300):
        mean_rep.append(0)

    for term in terms:
        rep = None
        try:
            rep = model[term]
        except KeyError:
            print >> sys.stderr, 'KEY ERROR FOR :', filename
            rep = []
            for i in range(0, 300):
                rep.append(0)


        reps[term] = rep
        for i in range(0, 300):
            mean_rep[i] += rep[i]

    if len(terms) > 0:
        for i in range(0, 300):
            mean_rep[i] = mean_rep[i] / len(terms)

    filename_last_part = filename.split('/')[-1]
    print filename
    with open(filename, 'r') as f, open('%s/%s' % (output_dir, filename_last_part), 'w') as w:
        for line in f:
            parts = line.strip().split('\t')
            qid = int(parts[-1])
            term = parts[-2]
            if not term.startswith('#'):
                feats = compute_rep_feat_mean(reps[term], mean_rep)
                feature_parts = line.strip().split('#', 1)
                true_recall = float(feature_parts[0].split(' ')[0])
                w.write('%f '% true_recall)
                feat_id = 1
                for value in feats:
                    w.write('%d:%f ' % (feat_id, value))
                    feat_id +=1

                w.write('#%s\n' % (feature_parts[1]))


                
            
    


