/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <string>
#include <cstdlib>
#include <ctime>

using namespace std; 

vector<string> split(const string &s, char delim) {
    vector<string> elems;
    stringstream ss(s);
    string item;
    while(getline(ss, item, delim)) {
        elems.push_back(item);
    }
    
    return elems;
}

int main(int argc, char **argv)
{
    if(argc != 4)
    {
        cout<<"Usage: "<< argv[0]<<" <SVM TEST> <bias> <query_output>"<<endl;
        exit(0);
    }
    
    ifstream svm_test(argv[1], ifstream::in);
    float bias = atof(argv[2]);
    ofstream query_output(argv[3], ofstream::out);
    
    string line;
    float p_recall;
    int pre_qid = -1;

    srand((unsigned)time(0));
    
    while(getline(svm_test, line))
    {
        vector<string> splits = split(line, ' ');
        int randint = rand();
        if(randint > RAND_MAX / 2)
            bias = - bias;
        p_recall = atof(splits[0].data()) + bias;
        //cout<<p_recall<<endl;
        int size = splits.size();
        int qid = atoi((splits[size-1]).c_str());
        string term = splits[size-2];
        if(qid != pre_qid)
        {
            if(pre_qid == -1)
            {
                query_output<<"<parameters>"<<endl;
            }
            else
            {
                query_output<<" )"<<endl;
                query_output<<"\t</text></query>"<<endl;
            }
            
            query_output<<"\t<query><type>indri</type><number>"<<qid<<"</number><text>"<<endl;
            query_output<<"#weight( ";
            pre_qid = qid;
        }
        
        // Original probability
        query_output<<p_recall<<" "<<term<<" ";
    }

    // Output last
    if(pre_qid!=-1)
    {
        query_output<<" )"<<endl;
        query_output<<"\t</text></query>"<<endl;
        query_output<<"</parameters>"<<endl;
    }

    query_output.close();
    svm_test.close();
    
    return 0;
}
