/*==========================================================================
 * Copyright (c) 20014 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <string>

using namespace std; 

vector<string> split(const string &s, char delim) {
    vector<string> elems;
    stringstream ss(s);
    string item;
    while(getline(ss, item, delim)) {
        elems.push_back(item);
    }
    
    return elems;
}

int main(int argc, char **argv)
{
    if(argc != 3)
    {
        cout<<"Usage: "<< argv[0]<<" <SVM TEST> <query_output>"<<endl;
        exit(0);
    }
    
    ifstream svm_test(argv[1], ifstream::in);
    ofstream query_output(argv[2], ofstream::out);
    
    string line;
    string p_recall;
    int pre_qid = -1;
    
    while(getline(svm_test, line))
    {
        vector<string> splits = split(line, '\t');
        p_recall = splits[0];
        //cout<<p_recall<<endl;
        int size = splits.size();
        int qid = atoi((splits[size-1]).c_str());
        string term = splits[size-2];
        if(qid != pre_qid)
        {
            if(pre_qid == -1)
            {
                query_output<<"<parameters>"<<endl;
            }
            else
            {
                query_output<<" )"<<endl;
                query_output<<"\t</text></query>"<<endl;
            }
            
            query_output<<"\t<query><type>indri</type><number>"<<qid<<"</number><text>"<<endl;
            query_output<<"#weight( ";
            pre_qid = qid;
        }
        
        
        float recall = atof(p_recall.data());
        // Original probability
        
        //query_output<<p_recall<<" "<<term<<" ";

        // Odds probability
        query_output<<recall/(recall+1.0)<<" "<<term<<" ";
    }

    // Output last
    if(pre_qid!=-1)
    {
        query_output<<" )"<<endl;
        query_output<<"\t</text></query>"<<endl;
        query_output<<"</parameters>"<<endl;
    }

    query_output.close();
    svm_test.close();
    
    return 0;
}
