from Constants import *
import subprocess
import os
import glob
import sys
from time import strftime

def train_svm(train_qids, fbdoc, svddim, run_tag, nsample=None):
    oddp_tag = '_oddp' if '.oddp' in run_tag else ''
    prob_tag = ' -b 1 ' if '.oddp' not in run_tag else ' '
    nsample_tag = '.nsample.%d' % nsample if not nsample == None else ''
    train_qids.sort()
    svmname = None
    if train_qids[-1]-train_qids[0]+1 != len(train_qids):
        for index in xrange(len(train_qids)):
            if train_qids[index]-train_qids[0]!=index:
                svmname = 'clueweb09.q%d-%d-%d.%d.desc.fbdoc%d.svddim%d%s%s'%(train_qids[0],train_qids[index],train_qids[-1], len(train_qids),fbdoc,svddim,run_tag,nsample_tag)
                break
    else:
        svmname = 'clueweb09.q%d-%d.%d.desc.fbdoc%d.svddim%d%s%s'%(train_qids[0],train_qids[-1], len(train_qids),fbdoc,svddim,run_tag, nsample_tag)
    
    # read training examples
    ofilename = '../tmp/%s.train.svm'%svmname
    ofile=open(ofilename,'w')
    for qid in train_qids:
        mu = clueweb09_mus[0]
        ifilename = ''
        if run_tag == '' or run_tag == '.oddp':
            ifilename = '../clueweb09_qtfs%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.svm'%(oddp_tag, qid, mu, fbdoc, svddim)
        elif run_tag == '.rf' or run_tag == '.rf.oddp':
            ifilename = '../qtfs%s/q%d.okw.desc.fbdocs%d.svddim%d.rf.svm'%(oddp_tag, qid, fbdoc, svddim)
        elif run_tag == '.resample' or run_tag == '.resample.oddp':
            ifilename = '../qtfs_linear%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.nsample.%d.svm'%(oddp_tag, qid, mu, fbdoc, svddim, nsample)
        elif run_tag == '.reweighted':
            ifilename = '../qtfs_linear%s/q%d.okw.desc.mu%d.fbdocs%d.svddim%d.reweighted.svm'%(oddp_tag, qid, mu, fbdoc, svddim)
        
        # query term feature file not exist

        if not os.path.exists(ifilename):
            continue
            #os.system('./query_term_feature_gen %d ../res/t%d.okw.mu%d.res %d %d %s'%(qid, trecid, mu, fbdoc, svddim, ifilename))
        
        ifile=open(ifilename,'r')
        for line in ifile:
            ofile.write('%s\n'%line.strip())

        ifile.close()

    ofile.close()

    #rescale train files
    os.system('../libsvm-3.14/svm-scale -l -u -s ../tmp/%s.rescale.param %s > %s.rescale' % (svmname, ofilename, ofilename))

    #train svm on condor
    jobstr = '%s.%s'%(svmname, strftime('%Y%m%d'))
    jobname = '.jobs/' + jobstr + '.job'
    f = open(jobname, 'w')
    f.write('Universe = vanilla\n')
    f.write('initialdir = /bos/usr2/gzheng/TermRecall/src\n')
    f.write('executable = /opt/python27/bin/python\n')
    f.write('arguments = ../libsvm-3.14/tools/gridregression.py -out ../out/%s.out %s %s.rescale\n'%(svmname, prob_tag, ofilename))
#    f.write('arguments = cv_regression.py %s.rescale ../out_L1/%s.out\n'%(ofilename, svmname))
    f.write('output = .jobs/'+jobstr+'.out\n')
    f.write('log = .jobs/'+jobstr+'.log\n')
    f.write('error = .jobs/'+jobstr+'.err\n')
    f.write('queue\n')
    f.close()
    os.system('condor_submit '+ jobname)    

def best_svm_performance(train_qids, test_qids, run_tag=''):
    if run_tag == '.resample' or run_tag == '.resample.oddp':
        for nsample in range(1,31):
            train_svm(train_qids, good_fbdocs[trecno], good_svddims[trecno], run_tag, nsample)
    else:
        for fbdoc in ([500] if run_tag == '.rf' or run_tag == '.rf.oddp' else clueweb09_fbdocs):
            #    for fbdoc in range(10,101,10):
            for svddim in clueweb09_svddims:
            #(svddims_resample if run_tag == '.resample' or run_tag == '.resample.oddp' else svddims):
                train_svm(train_qids, fbdoc, svddim, run_tag)

def call_main(argstr):
    args = argstr.split(' ')
    train_qids = []
    test_qids = []
    run_tag = ''
    
    for arg in args:
        if arg.startswith("TRAIN"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            train_qids += range(start_qid, end_qid+1)
        elif arg.startswith("TEST"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            test_qids += range(start_qid, end_qid+1)
        elif arg == 'ODDP':
            run_tag = '.oddp'
        elif arg == 'RF':
            run_tag = '.rf'
        elif arg == 'REWEIGHTED':
            run_tag = '.reweighted'
        elif arg == 'RESAMPLE':
            run_tag = '.resample'
        elif arg == 'RF.ODDP':
            run_tag = '.rf.oddp'
        elif arg == 'RESAMPLE.ODDP':
            run_tag = '.resample.oddp'
        else:
            print 'Train/Test dataset error!'
            exit(1)

    best_svm_performance(train_qids, test_qids, run_tag)

if __name__ == '__main__':
    if(len(sys.argv)<3):
        print 'Usage: %s TRAIN:<train_qids_range> TEST:<test_qids_range>' % sys.argv[0]
        print 'Sample: %s TRAIN:151-200 TEST:201-250'%sys.argv[0]
        exit(1)
        
    train_qids = []
    test_qids = []
    
    for arg in sys.argv[1:]:
        if arg.startswith("TRAIN"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            train_qids += range(start_qid, end_qid+1)
        elif arg.startswith("TEST"):
            start_qid = int(arg.split(':')[1].split('-')[0])
            end_qid = int(arg.split(':')[1].split('-')[1])
            test_qids += range(start_qid, end_qid+1)
        else:
            print 'Train/Test dataset error!'
            exit(1)

    best_svm_performance(train_qids, test_qids)
