#include "FeatureGenerator.h"
#include <indri/Index.hpp>
#include <vector>
#include <Eigen/Dense>
#include <sstream>

using namespace std;
using namespace indri::index;
using namespace indri::collection;
using namespace Eigen;

std::string str(int a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

std::string str(float a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

int qid2trecyear(int qid)
{
    return 2009 + (qid -1 ) / 50;
}

int main(int argc, char **argv)
{
    if(argc != 6)
    {
        cout<<"Usage: "<< argv[0]<<" <Query ID> <initial_res> <fbdocs> <svddim> <featurefle>"<<endl;
        exit(0);
    }

    int qid = atoi(argv[1]);
    std::string initial_res(argv[2]);
    int fbdocs = atoi(argv[3]);
    int svddim = atoi(argv[4]);
    
    std::ofstream data_out(argv[5], std::ofstream::out);
    std::vector<std::string> clueweb09_indexes;

    for(int i=1;i<=9;i++)
    {
        clueweb09_indexes.push_back("/bos/tmp9/indexes/ClueWeb09_Indexes/ClueWeb09_English_0" + str(i));
    }
    clueweb09_indexes.push_back("/bos/tmp9/indexes/ClueWeb09_Indexes/ClueWeb09_English_10");

    std::string qrel = "../clueweb09_qrels/qrels." + str(qid2trecyear(qid)) + ".adhoc";

    cout<<"Processing query: "<<qid<<" fbdocs: "<<fbdocs<<" svddim: "<<svddim<<endl;
    std::string dep_res = "../clueweb09_parses/" + str(qid) + ".dep";
    
    // Seems now do not need to set query str
    std::string query_str = "";
    FeatureGenerator fg(clueweb09_indexes, qid, query_str, initial_res);
    fg.generateTFIDFMatrix(fbdocs);
    fg.compute_concept_matrix(svddim);
    fg.load_qrel(qrel);
    fg.load_dep(dep_res);
    data_out<<fg.generate_svm_features();
    data_out.flush();
    data_out.close();
    
    return 0;
}
