import sys
import math
import numpy as np
from sklearn import svm, grid_search
from sklearn.datasets import load_svmlight_file

def range_f(begin,end,step):
    # like range, but works on non-integer too
    seq = []
    while True:
        if step > 0 and begin > end: break
        if step < 0 and begin < end: break
        seq.append(begin)
        begin = begin + step
    return seq

def exp_range(seq):
    exp_seq = []
    for e in seq:
        exp_seq.append(2**e)
    return exp_seq

def L1_loss(y1, y2):
    return np.mean(abs(y1-y2))

if __name__ == '__main__':
    if(len(sys.argv)<3):
        print 'Usage: %s <svmfile> <outfile>' % sys.argv[0]
        exit(1)
    
    c_begin, c_end, c_step = -1,  8, 1
    g_begin, g_end, g_step =  1, -1, -0.25
    p_begin, p_end, p_step =  -8, -1, 1

    X_train, y_train = load_svmlight_file(sys.argv[1])
    params = {'C':exp_range(range_f(c_begin, c_end, c_step)), 'gamma':exp_range(range_f(g_begin, g_end, g_step)) }
#    params = {'C':exp_range(range_f(c_begin, c_end, c_step)), 'gamma':exp_range(range_f(g_begin, g_end, g_step)), 'epsilon':[0.1]}
#    params = {'C':exp_range(range_f(c_begin, c_end, c_step)), 'gamma':exp_range(range_f(g_begin, g_end, g_step)), 'epsilon':exp_range(range_f(p_begin, p_end, p_step))}
    print params
    svr = svm.SVR(probability=True)
    sol = grid_search.GridSearchCV(svr, params, loss_func=L1_loss, cv = 5)
    sol.fit(X_train, y_train)
    best_c = math.log(sol.best_params_['C'], 2)
    best_g = math.log(sol.best_params_['gamma'], 2)
    best_score = -sol.best_score_
    ofile=open(sys.argv[2], 'w')
    ofile.write('%f %f %f\n'%(best_c, best_g, best_score))
    ofile.close()

