import os
import sys
import subprocess
from Constants import *
from doit_deep_cikm import prepare_svm_file
from doit_deep_cikm import get_all_qids

def get_trec_id(trec_str):
    if trec_str == 'robust04':
        return 7
    elif trec_str == 'wt10g':
        return 9
    elif trec_str == 'gov2':
        return 13
    elif trec_str == 'clueweb09b':
        return 21

if len(sys.argv)!=6:
    print 'Usage: %s <trec_str> <QL|SD> <LM|BM25> <qtfs_folder> <tmp_folder>' % sys.argv[0]
    exit(0)

trec_str = sys.argv[1]
trecid = get_trec_id(trec_str.strip())

query_model = sys.argv[2]
retrieval_model = sys.argv[3]

if (query_model!= 'QL' and query_model != 'SD') or (retrieval_model != 'LM' and retrieval_model !='BM25'):
    print 'Usage: %s <trec_str> <QL|SD> <LM|BM25> <qtfs_folder> <tmp_folder>' % sys.argv[0]
    exit(0)


qtfs_folder = sys.argv[4]
tmp_folder = sys.argv[5]

all_qids = get_all_qids(trec_str)


nfold = 5

foldsize = len(all_qids) / nfold

folds =[]
res_file = '%s/%s.res' % (tmp_folder, trec_str)
query_file = '%s/%s.q' % (tmp_folder, trec_str)

os.system(' > %s' % res_file)
os.system(' > %s' % query_file)

for i in range(0, nfold):
    test_qids = all_qids[i*foldsize:(i+1)*foldsize]
    train_qids = all_qids[:i*foldsize] + all_qids[(i+1)*foldsize:]

    train_file = '%s/%s.fold%d.train.svm' % (tmp_folder, trec_str, (i+1))
    test_file  = '%s/%s.fold%d.test.svm' %  (tmp_folder, trec_str, (i+1))

    prepare_svm_file(qtfs_folder, train_qids, 2500, 600, 50, train_file)
    prepare_svm_file(qtfs_folder, test_qids, 2500, 600, 50, test_file)

    os.system('/opt/python27/bin/python2.7 svm2mtb_complete.py %s' % (train_file))
    os.system('/opt/python27/bin/python2.7 svm2mtb_complete.py %s' % (test_file))

    #lasso regression with deep features
    command = '/opt/matlab/7.13/bin/matlab -nodesktop -nojvm -singleCompThread -nosplash -r \"deep_lasso(\'%s.A\', \'%s.y\', \'%s.A\', \'%s.y\')\"' %(train_file, train_file, test_file, test_file)

    os.system(command)

    prefix = '%s.%s' % (train_file, test_file.split('/')[-1])

    if query_model == 'QL' and retrieval_model == 'LM':
        # weighted keyword query
        os.system('/opt/python27/bin/python2.7 wqg_P_SIGIR.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1], prefix))

    elif query_model == 'SD' and retrieval_model == 'LM':
        # deep weighted sd - (unigrams weighted only)
        os.system('/opt/python27/bin/python2.7 wqg_P_mixed_deep.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1],  prefix))

    elif query_model == 'QL' and retrieval_model == 'BM25':
        # generate BM25 queries (P(t|R) needs to be absored in weight -> see BM25 retrieval fuction)
        os.system('/opt/python27/bin/python2.7 wqg_P_BM25.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1], prefix))

    elif query_model == 'SD' and retrieval_model == 'BM25':
        # deep weighted sd - (unigrams weighted only for BM25)
        os.system('/opt/python27/bin/python2.7 wqg_P_mixed_deep_BM25.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1],  prefix))


    if retrieval_model == 'LM':
        # indri 5.3 for LM
        os.system('/bos/usr0/gzheng/.local/bin/IndriRunQuery -count=1000 %s.%s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:d,mu:2500 > %s.res' %(train_file, test_file.split('/')[-1], trecid, prefix))

    elif retrieval_model == 'BM25':
        #indri 5.3 for BM25 
        os.system('/bos/usr0/gzheng/.local/bin/IndriRunQuery -count=1000 %s.%s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:okapi > %s.res' %(train_file, test_file.split('/')[-1], trecid, prefix))
        

    os.system('cat %s.res >> %s' % (prefix, res_file))
    os.system('cat %s.q >> %s' % (prefix, query_file))

#eval_output_test = subprocess.check_output(('./trec_eval ../qrels/qrels.t%d.txt %s.res' %(trecid, prefix)).split(' '))
#eval_map_test = float(eval_output_test.strip().split('\n')[5].split('\t')[-1])
#eval_p10_test = float(eval_output_test.strip().split('\n')[22].split('\t')[-1])

# significance test
qrel_cmd = './trec_eval -q ../qrels/qrels.%s.txt %s > %s.eval' % (trec_str, res_file, res_file)
os.system(qrel_cmd)

if retrieval_model == 'LM':
    sigtest_lm_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file lm.%s.eval %s.eval' % (trec_str, res_file)
    sigtest_sd_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file sd.%s.eval %s.eval' % (trec_str, res_file)

elif retrieval_model == 'BM25':
    sigtest_lm_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file bm25.%s.eval %s.eval' % (trec_str, res_file)
    sigtest_sd_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file bm25.%s.sd.eval %s.eval' % (trec_str, res_file)

sig_lm_output = subprocess.check_output(sigtest_lm_cmd.split(' '))
lm_output_lines = sig_lm_output.strip().split('\n')

eval_p10_test = float(lm_output_lines[1].split('\t')[-3])
eval_p10_test_lm_delta = float(lm_output_lines[1].split('\t')[-2])

eval_map_test = float(lm_output_lines[2].split('\t')[-3])
eval_map_test_lm_delta = float(lm_output_lines[2].split('\t')[-2])

siglevel_lm_p10 = float(lm_output_lines[1].split('\t')[-1])
siglevel_lm_map = float(lm_output_lines[2].split('\t')[-1])

sig_sd_output = subprocess.check_output(sigtest_sd_cmd.split(' '))
sd_output_lines = sig_sd_output.strip().split('\n')

eval_p10_test_sd_delta = float(sd_output_lines[1].split('\t')[-2])
eval_map_test_sd_delta = float(sd_output_lines[2].split('\t')[-2])

siglevel_sd_p10 = float(sd_output_lines[1].split('\t')[-1])
siglevel_sd_map = float(sd_output_lines[2].split('\t')[-1])

map_sig_str = ''
if siglevel_lm_map < 0.05:
    map_sig_str += '*'
if siglevel_sd_map < 0.05:
    map_sig_str += '$'

p10_sig_str = ''
if siglevel_lm_p10 < 0.05:
    p10_sig_str += '*'
if siglevel_sd_p10 < 0.05:
    p10_sig_str += '$'

print 'FINAL_RESULT: %s\t%s\tMAP: %.4f%s\tP@10: %.4f%s'%(train_file, test_file, eval_map_test, map_sig_str, eval_p10_test, p10_sig_str)
print 'FINAL_RESULT: %s\t%s\tMAP: (%.1f%%/%.1f%%)' %(train_file, test_file, eval_map_test_lm_delta, eval_map_test_sd_delta)
#print 'FINAL_RESULT: %s\t%s\tMAP delta LM: %.4f\tSigLevel: %.4f\tMAP delta SD: %f\tSigLevel: %.4f' %(train_file, test_file, (eval_map_test_lm_delta), (siglevel_lm_map), (eval_map_test_sd_delta), (siglevel_sd_map))
#print 'FINAL_RESULT: %s\t%s\tP@10 delta LM: %.4f\tSigLevel: %.4f\tP@10 delta SD: %f\tSigLevel: %.4f' %(train_file, test_file, (eval_p10_test_lm_delta), (siglevel_lm_p10), (eval_map_test_sd_delta), (siglevel_sd_p10))

# prepare for latex output
#ClueWeb100-\texttt{DeepTR-KQ}& 0.2379 & 0.4200 & $0.1970^*$ & 0.3440   & 0.2719 & 0.5620 & 0.0849 & 0.2697\\
#          & \textit{(+4.4/+4.4)} &  & \textit{(+17.6/+3.8)} &  &\textit{(+3.3/-0.2)} & & \textit{(+6.5/-4.0)} & \\


'''
map_lm_sym = ''
map_sd_sym = ''
if siglevel_lm_map <= 0.05:
    map_lm_sym = '^*'

if siglevel_sd_map <= 0.05:
    map_sd_sym = '_\dagger'

p10_lm_sym = ''
p10_sd_sym = ''

if siglevel_lm_p10 <= 0.05:
    p10_lm_sym = '^*'
    
if siglevel_sd_p10 <= 0.05:
    p10_sd_sym = '_\dagger'
    


print 'VECTOR-\textit{DeepTR-ID} & $%.4f%s%s$ & $%.4f%s%s$ & '
'''
