import os
import sys
import subprocess
from Constants import *

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'

def g(number):
    return bcolors.OKGREEN + number + bcolors.ENDC

def r(number):
    return bcolors.WARNING + number + bcolors.ENDC


if len(sys.argv)!=6:
    print 'Usage: %s <train_svm> <test_svm> <test_tid> <QL|SD> <LM|BM25>'
    exit(0)


train_file = sys.argv[1]
test_file = sys.argv[2]
trecid = int(sys.argv[3])
query_model = sys.argv[4]
retrieval_model = sys.argv[5]

if (query_model!= 'QL' and query_model != 'SD') or (retrieval_model != 'LM' and retrieval_model !='BM25'):
    print 'Usage: %s <train_svm> <test_svm> <test_tid> <QL|SD> <LM|BM25>'
    exit(0)

#lasso regression with deep features
command = '/opt/matlab/7.13/bin/matlab -nodesktop -nojvm -singleCompThread -nosplash -r \"deep_lasso(\'%s.A\', \'%s.y\', \'%s.A\', \'%s.y\')\"' %(train_file, train_file, test_file, test_file)

os.system(command)

prefix = '%s.%s' % (train_file, test_file.split('/')[-1])

if query_model == 'QL' and retrieval_model == 'LM':
    # weighted keyword query
    os.system('/opt/python27/bin/python2.7 wqg_P_SIGIR.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1], prefix))

elif query_model == 'SD' and retrieval_model == 'LM':
    # deep weighted sd - (unigrams weighted only)
    os.system('/opt/python27/bin/python2.7 wqg_P_mixed_deep.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1],  prefix))

elif query_model == 'QL' and retrieval_model == 'BM25':
    # generate BM25 queries (P(t|R) needs to be absored in weight -> see BM25 retrieval fuction)
    os.system('/opt/python27/bin/python2.7 wqg_P_BM25.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1], prefix))

elif query_model == 'SD' and retrieval_model == 'BM25':
    # deep weighted sd - (unigrams weighted only for BM25)
    os.system('/opt/python27/bin/python2.7 wqg_P_mixed_deep_BM25.py %s %s.A.%s.A.out ../queries/desc_parts_stopped %s.q' %(test_file, train_file, test_file.split('/')[-1],  prefix))


if retrieval_model == 'LM':
    # indri 5.3 for LM
    os.system('/bos/usr0/gzheng/.local/bin/IndriRunQuery -count=1000 %s.%s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:d,mu:2500 > %s.res' %(train_file, test_file.split('/')[-1], trecid, prefix))


elif retrieval_model == 'BM25':
    #indri 5.3 for BM25 
    os.system('/bos/usr0/gzheng/.local/bin/IndriRunQuery -count=1000 %s.%s.q -index=/bos/tmp17/gzheng/trec/indext%d-v5.3/ -trecFormat=t -rule=method:okapi > %s.res' %(train_file, test_file.split('/')[-1], trecid, prefix))

#eval_output_test = subprocess.check_output(('./trec_eval ../qrels/qrels.t%d.txt %s.res' %(trecid, prefix)).split(' '))
#eval_map_test = float(eval_output_test.strip().split('\n')[5].split('\t')[-1])
#eval_p10_test = float(eval_output_test.strip().split('\n')[22].split('\t')[-1])

# significance test
qrel_cmd = './trec_eval -q ../qrels/qrels.t%d.txt %s.res > %s.eval' % (trecid, prefix, prefix)
os.system(qrel_cmd)

if retrieval_model == 'LM':
    sigtest_lm_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file lmt%d.eval %s.eval' % (trecid, prefix)
    sigtest_sd_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file sdt%d.eval %s.eval' % (trecid, prefix)

elif retrieval_model == 'BM25':
    sigtest_lm_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file bm25t%d.eval %s.eval' % (trecid, prefix)
    sigtest_sd_cmd = '/opt/perl/bin/perl paired-randomization-test-v2.pl -t a -d sigtest_metric_file bm25t%d.sd.eval %s.eval' % (trecid, prefix)

sig_lm_output = subprocess.check_output(sigtest_lm_cmd.split(' '))
lm_output_lines = sig_lm_output.strip().split('\n')

eval_p10_test = float(lm_output_lines[1].split('\t')[-3])
eval_p10_test_lm_delta = float(lm_output_lines[1].split('\t')[-2])

eval_map_test = float(lm_output_lines[2].split('\t')[-3])
eval_map_test_lm_delta = float(lm_output_lines[2].split('\t')[-2])

siglevel_lm_p10 = float(lm_output_lines[1].split('\t')[-1])
siglevel_lm_map = float(lm_output_lines[2].split('\t')[-1])

sig_sd_output = subprocess.check_output(sigtest_sd_cmd.split(' '))
sd_output_lines = sig_sd_output.strip().split('\n')

eval_p10_test_sd_delta = float(sd_output_lines[1].split('\t')[-2])
eval_map_test_sd_delta = float(sd_output_lines[2].split('\t')[-2])

siglevel_sd_p10 = float(sd_output_lines[1].split('\t')[-1])
siglevel_sd_map = float(sd_output_lines[2].split('\t')[-1])

map_sig_str = ''
if siglevel_lm_map < 0.05:
    map_sig_str += '*'
if siglevel_sd_map < 0.05:
    map_sig_str += '$'

p10_sig_str = ''
if siglevel_lm_p10 < 0.05:
    p10_sig_str += '*'
if siglevel_sd_p10 < 0.05:
    p10_sig_str += '$'

print 'FINAL_RESULT: %s\t%s\tMAP: %.4f%s\tP@10: %.4f%s'%(train_file, test_file, eval_map_test, map_sig_str, eval_p10_test, p10_sig_str)
print 'FINAL_RESULT: %s\t%s\tMAP: (%.1f%%/%.1f%%)' %(train_file, test_file, eval_map_test_lm_delta, eval_map_test_sd_delta)
#print 'FINAL_RESULT: %s\t%s\tMAP delta LM: %.4f\tSigLevel: %.4f\tMAP delta SD: %f\tSigLevel: %.4f' %(train_file, test_file, (eval_map_test_lm_delta), (siglevel_lm_map), (eval_map_test_sd_delta), (siglevel_sd_map))
#print 'FINAL_RESULT: %s\t%s\tP@10 delta LM: %.4f\tSigLevel: %.4f\tP@10 delta SD: %f\tSigLevel: %.4f' %(train_file, test_file, (eval_p10_test_lm_delta), (siglevel_lm_p10), (eval_map_test_sd_delta), (siglevel_sd_p10))

# prepare for latex output
#ClueWeb100-\texttt{DeepTR-KQ}& 0.2379 & 0.4200 & $0.1970^*$ & 0.3440   & 0.2719 & 0.5620 & 0.0849 & 0.2697\\
#          & \textit{(+4.4/+4.4)} &  & \textit{(+17.6/+3.8)} &  &\textit{(+3.3/-0.2)} & & \textit{(+6.5/-4.0)} & \\


'''
map_lm_sym = ''
map_sd_sym = ''
if siglevel_lm_map <= 0.05:
    map_lm_sym = '^*'

if siglevel_sd_map <= 0.05:
    map_sd_sym = '_\dagger'

p10_lm_sym = ''
p10_sd_sym = ''

if siglevel_lm_p10 <= 0.05:
    p10_lm_sym = '^*'
    
if siglevel_sd_p10 <= 0.05:
    p10_sd_sym = '_\dagger'
    


print 'VECTOR-\textit{DeepTR-ID} & $%.4f%s%s$ & $%.4f%s%s$ & '
'''
