#include "FeatureGenerator.h"
#include <indri/Index.hpp>
#include <vector>
#include <Eigen/Dense>
#include <sstream>

using namespace std;
using namespace indri::index;
using namespace indri::collection;
using namespace Eigen;

std::string str(int a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

std::string str(float a)
{
    std::ostringstream oss;
    oss<<a;
    return oss.str();
}

int main(int argc, char **argv)
{
    if(argc != 10)
    {
        cout<<"Usage: "<< argv[0]<<" <topic ID> <initial_res> <dep> <qrel> <fbdocs> <svddim> <click_doc_file> <snippet_file> <feature_file>"<<endl;
        exit(0);
    }

    int qid = atoi(argv[1]);
    std::string query_str = "";
    std::string initial_res(argv[2]);
    std::string dep_res(argv[3]);
    std::string qrel(argv[4]);
    int fbdocs = atoi(argv[5]);
    int svddim = atoi(argv[6]);
    std::string doc_list_file = argv[7];
    std::string snippet_file = argv[8];
    std::ofstream data_out(argv[9], std::ofstream::out);

    std::vector<std::string> clueweb09_indexes;

    for(int i=1;i<=9;i++)
    {
        clueweb09_indexes.push_back("/bos/tmp9/indexes/ClueWeb09_Indexes/ClueWeb09_English_0" + str(i));
    }
    clueweb09_indexes.push_back("/bos/tmp9/indexes/ClueWeb09_Indexes/ClueWeb09_English_10");


    cout<<"Processing query: "<<qid<<" "<<initial_res<<" fbdocs: "<<fbdocs<<" svddim: "<<svddim<<endl;
    
    // Seems now do not need to set query str
    FeatureGenerator fg(clueweb09_indexes, qid, query_str, initial_res, false);
    fg.generateTFIDFMatrix(fbdocs);

    // Add clicks and snippets
    fg.load_clicks(doc_list_file, snippet_file);
    fg.reweight_by_snippets();

    fg.compute_concept_matrix(svddim);
    fg.load_qrel(qrel);
    fg.load_dep(dep_res);
    data_out<<fg.generate_svm_st_features();
    data_out.flush();
    data_out.close();
    
    return 0;
}
