#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <string>
#include <cmath>

using namespace std; 

vector<string> split(const string &s, char delim) {
    vector<string> elems;
    stringstream ss(s);
    string item;
    while(getline(ss, item, delim)) {
        elems.push_back(item);
    }
    
    return elems;
}

float logistic(float c, float k, float x)
{
    return 1.0/(1.0+exp(c*(-k*x+k-1)));
}

float solve_y(float idf, float k)
{
    float epsilon = 0.000001;
    float y0 = 0;
    float y1 = 1;
    int iter = 0;
    float y;
    float fy;
    
    do{
        iter++;

        y = (y0+y1)/2;
        fy = logistic(idf, k, y);
        
        if(abs(fy-y)<epsilon){
            //            cout<<"SOLVER SUCCEED!"<<endl;
            return 1-y;
        }
        else if(fy>y)
        {
            y0 = y;
        }
        else
        {
            y1 = y;
        }

        if(iter%10000==0)
        {
            cout<<"Iter:"<<iter<<" "<<abs(fy-y)<<" "<<idf<<endl;
            cout<<"Y0:"<<y0<<" Y1:"<<y1<<" "<<(y0+y1)/2<<endl;
            cout<<"Y:"<<y<<" f(Y):"<<fy<<endl;
        }
 
    }while(1);
}

int main(int argc, char **argv)
{
    if(argc != 3)
    {
        cout<<"Usage: "<< argv[0]<<" <SVM TEST> <query_output>"<<endl;
        exit(0);
    }
    
    ifstream svm_test(argv[1], ifstream::in);
    ofstream query_output(argv[2], ofstream::out);

    ifstream k_input("k.val", ifstream::in);
    
    string line;
    string p_recall;
    int pre_qid = -1;
    
    while(getline(svm_test, line))
    {
        vector<string> splits = split(line, ' ');
        p_recall = splits[0];
        //cout<<p_recall<<endl;
        int size = splits.size();
        int qid = atoi((splits[size-1]).c_str());
        float idf = atof(split(splits[1], ':')[1].c_str());

        float rcl = atof(p_recall.data());
        //float ccr = 1 + log(rcl/(1-rcl)) / idf;
        float ccr = 1+log(rcl/(1-rcl)) / idf;

        string k_str;
        k_input>>k_str;
        //float k = atof(k_str.data());
        float k=2.7;
        //float predicted_ccr = solve_y(idf,k);

        string term = splits[size-2];
        if(qid != pre_qid)
        {
            if(pre_qid == -1)
            {
                query_output<<"<parameters>"<<endl;
            }
            else
            {
                query_output<<" )"<<endl;
                query_output<<"\t</text></query>"<<endl;
            }
            
            query_output<<"\t<query><type>indri</type><number>"<<qid<<"</number><text>"<<endl;
            query_output<<"#weight( ";
            pre_qid = qid;
        }
        
        // Original probability
        //query_output<<p_recall<<" "<<term<<" ";
        query_output<<ccr<<" "<<term<<" ";
        //query_output<<predicted_ccr<<" "<<term<<" ";
    }

    // Output last
    if(pre_qid!=-1)
    {
        query_output<<" )"<<endl;
        query_output<<"\t</text></query>"<<endl;
        query_output<<"</parameters>"<<endl;
    }

    query_output.close();
    svm_test.close();
    
    return 0;
}
