#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <string>

#define EPSILON 0.01

using namespace std; 

vector<string> split(const string &s, char delim) {
    vector<string> elems;
    stringstream ss(s);
    string item;
    while(getline(ss, item, delim)) {
        elems.push_back(item);
    }
    
    return elems;
}

int main(int argc, char **argv)
{
    if(argc != 4)
    {
        cout<<"Usage: "<< argv[0]<<" <SVM TEST> <SVM OUTPUT> <query_output>"<<endl;
        exit(0);
    }
    
    ifstream svm_test(argv[1], ifstream::in);
    ifstream svm_output(argv[2], ifstream::in);
    ofstream query_output(argv[3], ofstream::out);
    
    string line;
    double p_recall = 0;
    int pre_qid = -1;
    
    while(getline(svm_test, line))
    {
        svm_output>>p_recall;
        //cout<<p_recall<<endl;

        
        //vector<string> splits = split(line, ' ');
        vector<string> splits = split(line, '\t');
        int size = splits.size();
        int qid = atoi((splits[size-1]).c_str());

        float sd_w = 0.6;
            
        string term = splits[size-2];

        if(term.find("#1(") != string::npos || term.find("#uw") != string::npos)
        {
            sd_w = 0.2;
        }
        
        
        if(qid != pre_qid)
        {
            if(pre_qid == -1)
            {
                query_output<<"<parameters>"<<endl;
            }
            else
            {
                query_output<<" )"<<endl;
                query_output<<"\t</text></query>"<<endl;
            }
            
            query_output<<"\t<query><type>indri</type><number>"<<qid<<"</number><text>"<<endl;
            query_output<<"#weight( ";
            pre_qid = qid;
        }
        
        // Odd probability to original probability
        if(p_recall < EPSILON)
            p_recall = EPSILON;
        
        double prob = p_recall/(p_recall + 1.0);
        query_output<<sd_w * prob<<" "<<term<<" ";
    }

    // Output last
    if(pre_qid!=-1)
    {
        query_output<<" )"<<endl;
        query_output<<"\t</text></query>"<<endl;
        query_output<<"</parameters>"<<endl;
    }

    query_output.close();
    svm_test.close();
    svm_output.close();
    
    return 0;
}
